//Time-stamp: <2004-03-26 18:50:47 mary>
/*
 *  
 Tutorial 2:

 Definition of a new robot (using the khepera class)
 usefull to add items like memory to the robots
 
 - We want to add a memory to the robot.
 - It will be used to store the path of the robot
 
 *
 */
//Classics headers
#include <iostream>
#include <vector>

//Environment, Robot,  Word and Controllers headers
#include "../src/heart/simulator.h"

// To use time to initialise Random number generator
#include <time.h>


//Definition of the type of memory
class Position
{
public:
  Position(Real & _x,Real & _y) : x(_x), y(_y)
  {}
  void display(){cout<<"("<<x<<","<<y<<")"<<endl;}
private:  
  Real x;
  Real y;
};


class Memory
{
public :
  Memory()
  {}
  void add_position(Real _x,Real _y)
  {
	vector_of_positions.push_back(new Position(_x,_y));
  }
  int number_of_positions()
  {
	return vector_of_positions.size();
  }
  
  
  void print_all_positions()
  {
	for (vector<Position*>::iterator it1=vector_of_positions.begin();it1!=vector_of_positions.end();it1++) 
		  {
			(*it1)->display();
		  }
  }
  ~Memory()
  {
	for (vector<Position*>::iterator it1=vector_of_positions.begin();it1!=vector_of_positions.end();it1++) 
		  {
			delete (*it1);
		  }
  }
  
private :
  vector<Position *> vector_of_positions;
};


// definition of our khepera with memory
class khepera_M: public khepera
{
public:
  // here are the 2 standards constructors of khepera - you can modify it but keep in mind that you have to call a constructor of khepera in each of your constructors
  khepera_M(string _namerobot, Controller * _controller_base, Real _radius=27.5) : khepera(_namerobot,_controller_base,_radius)
  {memory=new Memory();}
  khepera_M(string _namerobot, Controller * _controller_base, Real _centerx, Real _centery, Real _direction, Real _radius=27.5) : khepera(_namerobot,_controller_base,_centerx,_centery,_direction,_radius)
  {memory=new Memory();}



  void add_position(Real _x,Real _y)
  {
	memory->add_position(_x,_y);
  }
  void print_all_positions()
  {
	memory->print_all_positions();
  }
  int number_of_positions()
  {
	return memory->number_of_positions();
  }
  
  ~khepera_M() // destructor
  { //add here the destructors of the things that you added to your khepera
	delete memory;
	
	// you NEVER need to modify the destructor beyond this point (it is a call to the destructor of khepera)
	Partial_self_destruct();
  }
  
  
private:
  class Memory *  memory;
};


void simple_khepera_M_controller (robot & _myrobot) 
	 
{
  // This is the same than the previous controller except for memory
#ifdef _WIN32
  khepera_M * myrobot =(static_cast<khepera_M *>(& _myrobot));
#else
  khepera_M * myrobot =(dynamic_cast<khepera_M *>(& _myrobot));
#endif

  myrobot->add_position(myrobot->centerx,myrobot->centery);
  if (myrobot->number_of_positions()==50)
	{
	  myrobot->print_all_positions();
	}
  
  
  //if (myrobot->crash_obstacle) {cout<<"crash"<<endl; }
  
  
	  if ((myrobot->IR(2) < 20) && (myrobot->IR(3) < 20) &&!(myrobot->crash_obstacle) &&!(myrobot->crash_other_robot))  //If the front sensors are not activated and no choc
		{
		  myrobot->decision.left_motor=1.0; 
		  myrobot->decision.right_motor=1.0; // the robot goes forward at maximal speed 
		}
  
	  else  //If the front sensors are activated
		{
		  myrobot->decision.left_motor=1.0; 
		  myrobot->decision.right_motor=0.0; // the robot turn on the right at maximal speed
	
		}


};



#define WIDTH  500  
#define LENGHT 500  


extern world world_GL;  // this is the world used by the main function and the main Display function for openGL


int main (int argc, char *argv[]) // int argc, char *argv[ ], char *envp[ ] )
{
     
  //first of all : we think the name of function is expressive enough	 
  if (GL_world_initialisation(argc,argv)==1) {return 1; }//error with OpenGL
  
  
  //secondly, we add the parameters for the environment and for the robots
  try
	
	{
	  srand(time(NULL)); //random generator initialisation
	  
      environment env0(WIDTH,LENGHT);
      
      env0.add_wall("wall0",0.0,0.0,0.0,LENGHT,unsigned(255));
      env0.add_wall("wall1",0.0,0.0,WIDTH,0.0,unsigned(255));
      env0.add_wall("wall2",0.0,LENGHT,WIDTH,LENGHT,unsigned(255));
      env0.add_wall("wall3",WIDTH,0.0,WIDTH,LENGHT,unsigned(255));
      
      env0.add_cylinder_unfixed("cylinder_unfixed0",200.0,150.0,25.0,unsigned(255));
	  env0.add_bloc("bloc1",400.0,400.0,100.0,0.0,0.0,100.0,unsigned(255));

	  Controller *  ctrl=new Controller (simple_khepera_M_controller);
	  
	  all_the_controllers.add_a_controller("controller_khepera_M_simple",ctrl);
	  
	  
      khepera_M myfirstkhepera ("first",all_the_controllers("controller_khepera_M_simple"));
	  khepera_M mysecondkhepera ("second",all_the_controllers("controller_khepera_M_simple"),100.0,100.0,M_PI/2);
	  

	  world_GL.set_environment(env0);

	  //world_GL.add_robot(mysecondkhepera);
	  world_GL.add_robot(myfirstkhepera); // you should add before the robots with a fixed start.
	  

	  
	  // boucle principale de gestion des vnements (only if you whish to use OPENG_GL display)
	  glutMainLoop();
	  
	  return 0;
	  
		
	}
  
 
  catch (error & _error)
	
	{
	  _error.what();
	}
  
  return 0;
  
  }
  



