#ifndef GEOMARC2_H
#define GEOMARC2_H


#include "MagicFMLibType.h"
#include "GeomCircle2.h"
#include <string>
 using namespace std;

namespace Geom {


class MAGICFM Arc2 : public Circle2
{
public:
    // The arc is defined by two points End0 and End1 on the circle so that
    // End1 is obtained from End0 by traversing counterclockwise.  The
    // application is responsible for ensuring that End0 and End1 are on the
    // circle and that they are properly ordered.

    Arc2 ();
	string type() {return "Arc";}
	
	
    Vector2& End0 ();
    const Vector2& End0 () const;

    Vector2& End1 ();
    const Vector2& End1 () const;

    // Test if P is on the arc.  The application must ensure that P is on the
    // circle; that is, |P-C| = R.  This test wo_s regardless of the angle
    // between B-C and A-C.
    bool Contains (const Vector2& _Point) const;
	
	// rotations & translations
	virtual void Translate(Real _tx, Real _ty)
	  {
		m_kCenter.Translate(_tx,_ty);
		m_kEnd0.Translate(_tx,_ty);
		m_kEnd1.Translate(_tx,_ty);
	  }
	virtual void Rotate(Real _cost, Real _sint, Real _centerx, Real _centery)
	  {
		m_kCenter.Rotate(_cost,_sint,_centerx,_centery);
		m_kEnd0.Rotate(_cost,_sint,_centerx,_centery);
		m_kEnd1.Rotate(_cost,_sint,_centerx,_centery);
	  }
	virtual void Rotate(Real _theta, Real _centerx, Real _centery)
	  {
		m_kCenter.Rotate(_theta,_centerx,_centery);
		m_kEnd0.Rotate(_theta,_centerx,_centery);
		m_kEnd1.Rotate(_theta,_centerx,_centery);
	  }
	
protected:
    Vector2 m_kEnd0, m_kEnd1;
};

#include "GeomArc2.inl"

} // namespace Geom

#endif


