
#include "GeomIntr2DBoxBox.h"
using namespace Geom;

//----------------------------------------------------------------------------
bool Geom::TestIntersection (const Box2& _Box0, const Box2& _Box1)
{
    // convenience variables
    const Vector2* akA = _Box0.Axes();
    const Vector2* akB = _Box1.Axes();
    const Real* afEA = _Box0.Extents();
    const Real* afEB = _Box1.Extents();

    // compute difference of box centers, D = C1-C0
    Vector2 kD = _Box1.Center() - _Box0.Center();

    Real aafAbsAdB[2][2], fAbsAdD, fRSum;
    
    // axis C0+t*A0
    aafAbsAdB[0][0] = Math::FAbs(akA[0].Dot(akB[0]));
    aafAbsAdB[0][1] = Math::FAbs(akA[0].Dot(akB[1]));
    fAbsAdD = Math::FAbs(akA[0].Dot(kD));
    fRSum = afEA[0] + afEB[0]*aafAbsAdB[0][0] + afEB[1]*aafAbsAdB[0][1];
    if ( fAbsAdD > fRSum )
        return false;

    // axis C0+t*A1
    aafAbsAdB[1][0] = Math::FAbs(akA[1].Dot(akB[0]));
    aafAbsAdB[1][1] = Math::FAbs(akA[1].Dot(akB[1]));
    fAbsAdD = Math::FAbs(akA[1].Dot(kD));
    fRSum = afEA[1] + afEB[0]*aafAbsAdB[1][0] + afEB[1]*aafAbsAdB[1][1];
    if ( fAbsAdD > fRSum )
        return false;

    // axis C0+t*B0
    fAbsAdD = Math::FAbs(akB[0].Dot(kD));
    fRSum = afEB[0] + afEA[0]*aafAbsAdB[0][0] + afEA[1]*aafAbsAdB[1][0];
    if ( fAbsAdD > fRSum )
        return false;

    // axis C0+t*B1
    fAbsAdD = Math::FAbs(akB[1].Dot(kD));
    fRSum = afEB[1] + afEA[0]*aafAbsAdB[0][1] + afEA[1]*aafAbsAdB[1][1];
    if ( fAbsAdD > fRSum )
        return false;

    return true;
}
//----------------------------------------------------------------------------


