#ifndef GEOMVECTOR2_H
#define GEOMVECTOR2_H

#include "GeomMath.h"
#include <math.h>
#include <string>
using namespace std;

namespace Geom {


  class MAGICFM Vector2 //: public PrimitivForm
{
public:
    // construction
    Vector2 ();
    Vector2 (Real fX, Real fY);
    Vector2 (Real afCoordinate[2]);
    Vector2 (const Vector2& _Vector);

	string type() {return "Vector";}
		
	
    // coordinates
    Real x, y;

    // access vector V as V[0] = V.x, V[1] = V.y
    //
    // WARNING.  These member functions rely on
    // (1) Vector2 not having virtual functions
    // (2) the data packed in a 2*sizeof(Real) memory block
    Real& operator[] (int i) const;
    operator Real* ();

    // assignment
    Vector2& operator= (const Vector2& _Vector);

	// rotations & translations
	void Translate(Real _tx, Real _ty);
	void Rotate(Real _cost, Real _sint, Real _centerx, Real _centery);
	void Rotate(Real _theta, Real _centerx, Real _centery);
	
    // comparison (supports fuzzy arithmetic when FUZZ > 0)
    bool operator== (const Vector2& _Vector) const;
    bool operator!= (const Vector2& _Vector) const;
    bool operator<  (const Vector2& _Vector) const;
    bool operator<= (const Vector2& _Vector) const;
    bool operator>  (const Vector2& _Vector) const;
    bool operator>= (const Vector2& _Vector) const;

    // arithmetic operations
    Vector2 operator+ (const Vector2& _Vector) const;
    Vector2 operator- (const Vector2& _Vector) const;
    Vector2 operator* (Real fScalar) const;
    Vector2 operator/ (Real fScalar) const;
    Vector2 operator- () const;
    MAGICFM friend Vector2 operator* (Real fScalar, const Vector2& _Vector);

    // arithmetic updates
    Vector2& operator+= (const Vector2& _Vector);
    Vector2& operator-= (const Vector2& _Vector);
    Vector2& operator*= (Real fScalar);
    Vector2& operator/= (Real fScalar);

    // vector operations
    Real Length () const;
    Real SquaredLength () const;
    Real Dot (const Vector2& _Vector) const;
    Real Unitize (Real fTolerance = 1e-06f);
    Vector2 Cross () const;  // returns (y,-x)
    Vector2 UnitCross () const;  // returns (y,-x)/sqrt(x*x+y*y)

    // Gram-Schmidt orthonormalization.
    static void Orthonormalize (Vector2 akVector[/*2*/]);

    // special points
    static const Vector2 ZERO;
    static const Vector2 UNIT_X;
    static const Vector2 UNIT_Y;

    // fuzzy arithmetic (set FUZZ > 0 to enable)
    static Real FUZZ;
};

#include "GeomVector2.inl"

} // namespace Geom

#endif


